const vec3 skyTopColor = vec3(0.02, 0.1, 0.4);
const vec3 skyAddColor = vec3(0.2, 0.1, 0.1);

uniform sampler2D gaux4;
uniform float frameTimeCounter;

// float tri(in float x)
// {
//     return clamp(abs(fract(x) - 0.5), 0.01, 0.49);
// }

// vec2 tri2(in vec2 p)
// {
//     return vec2(tri(p.x) + tri(p.y), tri(p.y + tri(p.x)));
// }

// mat2 mm2(in float a)
// {
//     float c = cos(a);
//     float s = sin(a);
//     return mat2(c,s,-s,c);
// }

// mat2 m2 = mat2(0.95534, 0.29552, -0.29552, 0.95534);

// float triNoise2d(in vec2 p, float spd)
// {
//     float z = 1.8;
//     float z2 = 2.5;
// 	float rz = 0.0;
//     p *= m2;
//     vec2 bp = p;
// 	for (float i = 0.0;i < 5.0; i++)
// 	{
//         vec2 dg = tri2(bp * 1.85) * 0.75;
//         dg *= m2;
//         p -= dg / z2;

//         bp *= 1.3;
//         z2 *= 0.45;
//         z *= 0.42;
// 		p *= 1.21 + (rz - 1.0) * 0.02;
        
//         rz += tri(p.x + tri(p.y)) * z;
//         p *= -m2;
// 	}
//     return clamp(1.0 / pow(rz * 29.0, 1.3), 0.0, 0.55);
// }

// vec4 aurora(vec3 ro, vec3 rd, float dither)
// {
//     vec4 col = vec4(0.0);
//     vec4 avgCol = vec4(0.0);
//     float h = (rd.y * 2.0 + 0.4);
//     for(float i = 0.0;i < 25; i++)
//     {
//         float of = 0.024 * dither * smoothstep(0.0, 15.0, i);
//         float pt = ((0.8 + pow(i, 1.4) * 0.006) - ro.y) / h - of;
//     	vec3 bpos = ro + pt * rd;
//         vec2 p = bpos.zx;
//         float rzt = triNoise2d(p, 0.06);
//         vec4 col2 = vec4(0.0, 0.0, 0.0, rzt);
//         col2.rgb = (sin(1.0 - vec3(2.15, -0.5, 1.2) + i * 0.043) * 0.5 + 0.5) * rzt;
//         avgCol =  (avgCol + col2) / 2;
//         col += avgCol * exp2(-i * 0.065 - 2.5) * smoothstep(0.0, 5.0, i);
//     }
//     col *= clamp(rd.y * 15.0 + 0.4, 0.0, 1.0);
    
//     return col * 1.8;
// }


vec3 starMap(in vec2 p, const float offset)
{
	const float C = 65536.0 / 3.14159265;
	vec4 v = (1.0 / 65536.0) * p.xyxy + vec4(0.1, 0.2, 0.3, 0.4);

	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);

	vec4 f = smoothstep(offset, 1.0, v);
    return vec3(1.0, 0.5, 0.0) * f.r + vec3(0.4, 0.0, 1.0) * f.g + vec3(0.0, 0.5, 1.0) * f.b + vec3(0.5, 1.0, 0.0) * f.w;
}

vec3 calcStarColor(in vec2 p, const float offset)
{
    vec4 ip = vec4(floor(p), ceil(p));
    vec2 fp = smoothstep(0.0, 1.0, fract(p));
    vec3 rf[4] = vec3 [](starMap(ip.xy, offset), starMap(ip.zy, offset), starMap(ip.xw, offset), starMap(ip.zw, offset));
    return mix(mix(rf[0], rf[1], fp.x), mix(rf[2], rf[3], fp.x), fp.y);
}

vec3 DrawAurora(in vec3 worldDir)
{
    vec2 fixedPos = worldDir.xz / (worldDir.y * 0.7 + 0.3);
    float samples = texture(gaux4, (fixedPos + frameTimeCounter * 0.03125) * 0.3).x;
    float R = exp(samples * -12.0);
    float G = exp(samples * -6.0);
    float B = exp(samples * -3.0);
    return vec3(R, G, B);
}

vec3 DrawNebula(in vec3 worldDir)
{
    vec3 shadowDir = shadowMat3 * worldDir;
    vec2 dir = normalize(shadowDir.xy);
    float cosf = cos(shadowDir.z * 4.0);
    float sinf = sin(shadowDir.z * 4.0);
    return calcStarColor(mat2(cosf, sinf, -sinf, cosf) * dir * 8.0, 0.0) * exp((shadowDir.z - 1.0) * 12.0) * 0.5 + exp((shadowDir.z - 1.0) * 256.0) * 2.0;
}


vec3 DrawEndSky(in vec3 worldDir, in float dither, const bool fog)
{
    vec3 col = skyTopColor + skyAddColor * pow2(min(1.0 - worldDir.y, 1.0));
    if(fog && worldDir.y > 0.0)
    {
        float fade = 1.0 - pow3(min(1.0 - worldDir.y, 1.0));
        col += DrawAurora(worldDir) * fade;
        col += calcStarColor(worldDir.xz / (worldDir.y * 0.5 + 0.5) * 300.0, 0.995) * fade;
        col += DrawNebula(worldDir) * fade;
    }
    return col;
}