
const float sunPhase = 0.04;
const float moonPhase = 0.035;
const float sunScattingOffset = -15.0;
const float moonScattingOffset = -20.0;

vec3 starMap(in vec2 p, const float offset)
{
	const float C = 65536.0 / 3.14159265;
	vec4 v = (1.0 / 65536.0) * p.xyxy + vec4(0.1, 0.2, 0.3, 0.4);

	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);

	vec4 f = smoothstep(offset, 1.0, v);
    return vec3(0.0, 0.75, 1.0) * f.r + vec3(1.0) * f.g + vec3(0.0, 0.25, 1.0) * f.b + vec3(0.5, 1.0, 1.0) * f.w;
}

vec3 calcStarColor(in vec2 p, const float offset)
{
    vec4 ip = vec4(floor(p), ceil(p));
    vec2 fp = smoothstep(0.0, 1.0, fract(p));
    vec3 rf[4] = vec3 [](starMap(ip.xy, offset), starMap(ip.zy, offset), starMap(ip.xw, offset), starMap(ip.zw, offset));
    return mix(mix(rf[0], rf[1], fp.x), mix(rf[2], rf[3], fp.x), fp.y);
}

vec3 DrawSky(in vec3 normpos, in vec3 sunVector, in bool sun)
{
	float skyline = pow2(min(1.0 - normpos.y, 1.0));
	
	float sunFlag = dot(normpos, sunVector);
	float moonFlag = -sunFlag;
	float sunmoonFlag = abs(sunFlag);
	
    float sflag = sunFlag * flag_day + moonFlag * flag_night;
	
    vec3 skycolor = skyTopColor * (max(sflag * 0.7 + 0.3, 0.0) * 0.3 + 0.7);
        
    float fade = min(abs(sunVector.y) * 10.0, 1.0);
    
    float skylinecurrent = (0.1 + 0.9 * fade * pow2(max(dot(normalize(normpos.xz), sunVector.xz * sign(sunVector.y)) * 0.5 + 0.5, 0.0)));
    skylinecurrent = mix(skylinecurrent, 1.0, smoothstep(0.0, -0.75, normpos.y) * rainStrength);
    
    float darkness = pow4(min(normpos.y + 1.0, 1.0));
    darkness = darkness * darkness * (3.0 - 2.0 * darkness);
    darkness = darkness * (1.0 - rainStrength) + rainStrength;
    
    skylinecurrent *= darkness;
    skycolor *= darkness * 0.7 + 0.3;
    skycolor += skyAddColor * skyline * skylinecurrent;

    float seed0 = distance(normpos, sunVector);
    float seed1 = distance(normpos, -sunVector);
    float scatray = exp2(max(seed0 - sunPhase, 0.0) * sunScattingOffset) * (1.0 - flag_night) + exp2(max(seed1 - moonPhase, 0.0) * moonScattingOffset) * flag_night;
    skycolor += skySunColor * scatray * pow4(darkness);
    
    if(sun)
    {
        skycolor += float(seed0 < sunPhase || seed1 < moonPhase) * pow4(darkness);
        float dist = 1.0 / normpos.y;
        vec2 fixcoord = normpos.xz * dist;
        fixcoord /= dist * 0.5 + 0.5;
        fixcoord += frameTimeCounter / 128.0;
        skycolor += calcStarColor(fixcoord * 235.0, 0.996) * (1.0 - pow2(skyline)) * flag_night * (1.0 - rainStrength);
    }
    
    
    return skycolor;
}