uniform mat4 gbufferModelView;
uniform mat4 gbufferProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;
uniform mat4 shadowModelViewInverse;
uniform mat4 shadowProjectionInverse;
uniform mat4 gbufferPreviousModelView;
uniform mat4 gbufferPreviousProjection;
uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;

#define shadowMapBias 0.885//[0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95]

vec3 projd = vec3(gbufferProjection[0].x, gbufferProjection[1].y, gbufferProjection[2].z);
vec3 prevprojd = vec3(gbufferPreviousProjection[0].x, gbufferPreviousProjection[1].y, gbufferPreviousProjection[2].z);
vec3 shadowprojd = vec3(shadowProjection[0].x, shadowProjection[1].y, shadowProjection[2].z);
vec4 iProjDiag = vec4(gbufferProjectionInverse[0].x, gbufferProjectionInverse[1].y, gbufferProjectionInverse[2].zw);

mat3 viewMat3 = mat3(gbufferModelView);
mat3 viewMat3Inverse = mat3(gbufferModelViewInverse);
mat3 viewMat3Prev = mat3(gbufferPreviousModelView);

mat3 shadowMat3 = mat3(shadowModelView);

vec3 getProj(in vec2 texcoord, in float depth)
{
    return vec3(texcoord, depth) * 2.0 - 1.0;
}

vec3 Proj2View(in vec3 p)
{
    vec4 fragposition = iProjDiag * p.xyzz + gbufferProjectionInverse[3];
    return fragposition.xyz / fragposition.w;
}

vec3 getView(in vec2 texcoord, const int depthID)
{
    float depth;
    if(depthID == 0)
        depth = texture(depthtex0, texcoord).x;
    else
        depth = texture(depthtex1, texcoord).x;
    
    depth += float(depth < 0.56) * 0.38;
    return Proj2View(getProj(texcoord, depth));
}

vec3 View2Proj(in vec3 p)
{
    return (projd * p + gbufferProjection[3].xyz) / -p.z;
}

vec3 View2World(in vec3 p)
{
    return viewMat3Inverse * p + gbufferModelViewInverse[3].xyz;
}

vec3 World2View(in vec3 p)
{
    return viewMat3 * p + gbufferModelView[3].xyz;
}

vec3 PreWorld2PreView(in vec3 p)
{
    return viewMat3Prev * p + gbufferPreviousModelView[3].xyz;
}

vec3 PreView2PreProj(in vec3 p)
{
    return (prevprojd * p + gbufferPreviousProjection[3].xyz) / -p.z;
}

vec3 World2Shadow(in vec3 p)
{
    return shadowMat3 * p + shadowModelView[3].xyz;
}

vec3 Shadow2ShadowProj(vec3 p)
{
    p = shadowprojd * p + shadowProjection[3].xyz;
    p.xy /= length(p.xy) * shadowMapBias + 1.0 - shadowMapBias;
    return p * 0.5 + 0.5;
}

vec3 World2ShadowProj(vec3 p)
{
    p = shadowMat3 * p + shadowModelView[3].xyz;
    p = shadowprojd * p + shadowProjection[3].xyz;
    p.xy /= length(p.xy) * shadowMapBias + 1.0 - shadowMapBias;
    return p * 0.5 + 0.5;
}