const vec3 skyTopColor = vec3(0.02, 0.1, 0.3);
const vec3 skyAddColor = vec3(0.45, 0.15, 0.0);

uniform sampler2D gaux4;
uniform float frameTimeCounter;

vec3 starMap(in vec2 p, const float offset)
{
	const float C = 65536.0 / 3.14159265;
	vec4 v = (1.0 / 65536.0) * p.xyxy + vec4(0.1, 0.2, 0.3, 0.4);

	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);

	vec4 f = smoothstep(offset, 1.0, v);
    return vec3(1.0, 0.5, 0.0) * f.r + vec3(0.4, 0.0, 1.0) * f.g + vec3(0.0, 0.5, 1.0) * f.b + vec3(0.5, 1.0, 0.0) * f.w;
}

vec3 calcStarColor(in vec2 p, const float offset)
{
    vec4 ip = vec4(floor(p), ceil(p));
    vec2 fp = smoothstep(0.0, 1.0, fract(p));
    vec3 rf[4] = vec3 [](starMap(ip.xy, offset), starMap(ip.zy, offset), starMap(ip.xw, offset), starMap(ip.zw, offset));
    return mix(mix(rf[0], rf[1], fp.x), mix(rf[2], rf[3], fp.x), fp.y);
}

vec3 TwilightForestSky(in vec3 worldDir, in float dither, const bool fog)
{
    vec3 col = skyTopColor + skyAddColor * pow2(min(1.0 - worldDir.y, 1.0));
    if(fog && worldDir.y > 0.0)
    {
        float fade = 1.0 - pow3(min(1.0 - worldDir.y, 1.0));
        col += calcStarColor(worldDir.xz / (worldDir.y * 0.5 + 0.5) * 300.0, 0.995) * fade;
        
    }
    
    return col;
}

#define VOLUME_RAY
#define VolumeRayMaxSteps 8//[5 6 7 8 9 10 11 12 13 14 15 16 18 24 32]
#define VolumeRayDistance 64.0//[8.0 16.0 24.0 32.0 48.0 56.0 64.0 72.0 80.0]
#define VolumeRayTotalStrength 1.0//[0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9]

#ifdef VOLUME_RAY
float TwilightForestVolumeRays(vec3 shadowDir, in float dist, in float dither, in bool sky)
{
    //float visiable = dot(shadowDir, -shadowModelView[3].xyz);
    float visiable = max(-shadowDir.y, 0.0);
    if(visiable > 0.0 && !sky)
    {
        float stepLen = VolumeRayDistance / VolumeRayMaxSteps;
        float len = stepLen * dither;
        int steps = min(int((dist - len) / stepLen + 0.0001), VolumeRayMaxSteps);

        shadowDir = shadowMat3 * shadowDir;

        vec3 testpoint = len * shadowDir + shadowModelView[3].xyz;
        vec3 teststep = shadowDir * stepLen;

        float sumResult = 0.0;

        for(int i = 0;i < steps;i++)
        {
            testpoint += teststep;
            sumResult += textureLod(shadowtex1, Shadow2ShadowProj(testpoint), 1.0) * stepLen;
        }
        return min(sumResult * 0.03125, 2.0) * visiable;
    }else{
        return visiable * 2.0;
    }
}
#endif