uniform sampler2D gaux4;
uniform float frameTimeCounter;

const int gaux4tureResolution = 512;

#define VOLUME_BLOCK_CLOUDS

#define CloudBottomHeight 400.0//[300.0 320.0 340.0 360.0 380.0 400.0 420.0 440.0 460.0 480.0 500.0]
#define CloudThickness 120.0//[50.0 60.0 70.0 80.0 90.0 100.0 110.0 120.0 130.0 140.0 150.0]

const float CloudAlpha = 0.01125;

#ifdef VOLUME_BLOCK_CLOUDS
float GetDensity(in vec3 position)
{
    return position.y > CloudBottomHeight && position.y < CloudBottomHeight + CloudThickness ? max(texelFetch(gaux4, ivec2(mod(position.xz / 160.0f, 512.0)), 0).g * 256.0 - 255.0, 0.0) : 0.0;
}

vec2 volumeCloudRender(in vec3 worldDir, in vec3 worldLightVec, in float dither, const int raySteps, const int shadowSteps)
{
    float fdistance = 1.0 / worldDir.y;
    float rayLength = CloudThickness / raySteps * fdistance;
    
    vec3 testPoint = vec3(frameTimeCounter * 4.0 + cameraPosition.x * 2.0, 0.0, frameTimeCounter * 8.0 + cameraPosition.z * 2.0);
    vec3 rayDirection = worldDir;
    
    float shadowLength = (CloudThickness / max(worldLightVec.y, 0.25)) / shadowSteps;
    vec3 shadowDir = worldLightVec * shadowLength;
    testPoint += rayDirection * fdistance * CloudBottomHeight;
    
    testPoint += rayDirection * rayLength * dither;
    float sumCloudDensity = 0.0;
    float cloudShadow = 1.0;
    

    float calcCloudAlpha = rayLength * CloudAlpha;
    float calcShadowStrength = shadowLength * 0.0104166;
    vec3 shadowStartWav = shadowDir * dither;

    for(int i = 0;i < raySteps && sumCloudDensity < 1.0;i++)
    {
        testPoint += rayDirection * rayLength;
        float sampleCloudDensity = GetDensity(testPoint);
        
        if(sampleCloudDensity > 0.0)
        {
            vec3 shadowtest = testPoint + shadowStartWav;
            float shad = 1.0;
            for(int j = 0;j < shadowSteps && shad > 0.0;j++)
            {
                shadowtest += shadowDir;
                shad -= GetDensity(shadowtest) * calcShadowStrength;
            }
            cloudShadow = mix(max(shad, 0.0), cloudShadow, min(sumCloudDensity, 1.0));
        }
        
        sumCloudDensity += sampleCloudDensity * calcCloudAlpha;
    }
    float finalBase = smoothstep(0.0, 1.0, sumCloudDensity);
    float edgeOfShadow = pow4(finalBase);
    float finalShadow = smoothstep(0.0, 1.0, cloudShadow) * edgeOfShadow + (1.0 - edgeOfShadow);
    return vec2(finalBase * smoothstep(60.0, 0.0, fdistance), finalShadow);
}
#endif

/*
vec4 maraabb(vec3 o, vec3 v, vec3 a, vec3 b)
{

	vec3 ta = (a - o) / v; //1 -  1 rcp  1 *
	vec3 tb = (b - o) / v; //1 -  1 rcp  1 *

	vec3 t2 = max(ta, tb);

	vec4 mint2 = vec4(min(min(t2.x, t2.y), t2.z));
	mint2.xyz = (step(vec3(0.0), v) * 2.0 - 1.0) * step(t2 - mint2.xyz, vec3(0.00001));

	return mint2;
}


float closamsun(vec3 sampos, vec3 aabbsam, vec3 sunvec, const float closamnum, const vec3 CLOSIZ)
{
	float samsum = 0.0;
	for (float k = 0.0; k < closamnum && sampos.y > CLOUD_Y && sampos.y < (CLOUD_Y + CLOUD_H); k++)
	{
		vec4 intsun = maraabb(sampos, sunvec, aabbsam, aabbsam + CLOSIZ);
		if (iscloud(aabbsam))
			samsum += intsun.w;
		sampos += intsun.w * sunvec;
		aabbsam += intsun.xyz * CLOSIZ;
	}
	return samsum;
}
*/