#version 460 core

out vec4 texBuffer3;

const float eyeBrightnessHalflife = 10.0f;

uniform int frameCounter;
uniform int isEyeInWater;
uniform ivec2 eyeBrightnessSmooth;
uniform float flag_day;
uniform float flag_night;
uniform float flag_sunset;
uniform float frameTimeCounter;
uniform float far;
uniform vec3 sunVec;
uniform vec3 moonVec;
uniform vec3 shadowVec;
uniform vec3 upVec;
uniform vec3 fogColor;
uniform vec3 skyTopColor;
uniform vec3 skyAddColor;
uniform vec3 skySunColor;
uniform vec3 waterFogColor;
uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;

in vec2 Texcoord;

#include "/HmwsLib/BasicFunctions.glsl"
#include "/HmwsLib/PackDatas.glsl"
#include "/HmwsLib/SpaceTransform.glsl"

vec2 hash22(in vec2 p)
{
	const float C = 65536.0 / 3.14159265;
	vec4 v = (1.0 / 65536.0) * p.xyxy + vec4(0.1, 0.2, 0.3, 0.4);

	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);

	return v.xy * 2.0 - 1.0;
}

vec2 randPos(in vec3 p)
{
    float cosz = cos(p.z);
    float sinz = sin(p.z);
    mat2 rot = mat2(cosz, sinz, -sinz, cosz);
    vec2 v = rot * hash22(p.xy);
    return v * 0.5 + 0.5;
}

float worleyNoise(in vec2 uv, in float time)
{
    vec2 index = floor(uv);
	vec2 pos = fract(uv);
    
	float d = 1.5;
    d = min(length(randPos(vec3(index + vec2(1.0), time)) + vec2(1.0) - pos), d);
    d = min(length(randPos(vec3(index + vec2(0.0, 1.0), time)) + vec2(0.0, 1.0) - pos), d);
    d = min(length(randPos(vec3(index + vec2(-1.0, 1.0), time)) + vec2(-1.0, 1.0) - pos), d);
    d = min(length(randPos(vec3(index + vec2(1.0, 0.0), time)) + vec2(1.0, 0.0) - pos), d);
    d = min(length(randPos(vec3(index, time)) - pos), d);
    d = min(length(randPos(vec3(index + vec2(-1.0, 0.0), time)) + vec2(-1.0, 0.0) - pos), d);
    d = min(length(randPos(vec3(index + vec2(1.0, -1.0), time)) + vec2(1.0, -1.0) - pos), d);
    d = min(length(randPos(vec3(index + vec2(0.0, -1.0), time)) + vec2(0.0, -1.0) - pos), d);
    d = min(length(randPos(vec3(index + vec2(-1.0), time)) + vec2(-1.0) - pos), d);

	return pow4(d);
}


void main()
{
    vec4 outColor = texture(colortex3, Texcoord);

	int materialID = int(texture(colortex2, Texcoord).y * 255.0 + 0.0001);

    vec3 viewPos0 = getView(Texcoord, 0);
    vec3 viewPos1 = getView(Texcoord, 1);

    float worldLen = length(viewPos0);
    float worldLen1 = length(viewPos1);

    outColor.rgb *= 1.0 + float(materialID == 4);

    if((materialID == 4 && isEyeInWater == 0) || (materialID != 4 && isEyeInWater == 1))
        outColor.rgb *= worleyNoise(View2World(viewPos1).xz + cameraPosition.xz + frameTimeCounter * 0.7, frameTimeCounter * 1.5) * 0.888 + 0.5;

    float distFact[3] = float[3] (materialID == 4 ? worldLen1 - worldLen : 0.0, worldLen, worldLen);
    vec3 waterFog[3] = vec3[3] (waterFogColor, waterFogColor, pow(fogColor, vec3(2.2)));
    outColor.rgb *= mix(normalize(waterFog[isEyeInWater]), vec3(1.0), exp(distFact[isEyeInWater] * -0.167));
    outColor.rgb = mix(waterFog[isEyeInWater] * float(eyeBrightnessSmooth.y) / 240.0, outColor.rgb, exp(distFact[isEyeInWater] * -0.111));
    
    texBuffer3 = outColor;
}
/* DRAWBUFFERS:3 */