#version 460 core

layout(location = 0) in vec4 TEXCOORD;

uniform float flag_day;
uniform float flag_night;
uniform float flag_sunset;

uniform float rainStrength;
uniform float rainday;
uniform float rainnight;
uniform vec2 screenTexel;
uniform vec3 sunVec;
uniform vec3 shadowVec;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform sampler2D depthtex0;

out vec2 Texcoord;

flat out float volumeRayStrength;
flat out vec3 worldSunVec;
flat out vec3 worldLightVec;

#define LENSFLARE

#ifdef LENSFLARE
flat out float sunVisibility;
flat out vec2 lf1Pos;
flat out vec2 lf2Pos;
flat out vec2 lf3Pos;
flat out vec2 lf4Pos;
#endif

#define LF1POS -0.3
#define LF2POS 0.2
#define LF3POS 0.7
#define LF4POS 0.75
 
void main()
{
    gl_Position = vec4(TEXCOORD.xy * 2.0 - 1.0, 0.0, 1.0);
    Texcoord = TEXCOORD.xy;
    volumeRayStrength = 0.2 * flag_day + 0.8 * flag_night + 0.75 * flag_sunset;
	
    mat3 viewInverse = mat3(gbufferModelViewInverse);
    worldSunVec = viewInverse * sunVec;
    worldLightVec = viewInverse * shadowVec;

    #ifdef LENSFLARE
    vec4 ndcSunPosition = gbufferProjection * vec4(shadowVec, 1.0);
    ndcSunPosition /= ndcSunPosition.w; 
    sunVisibility = 0.0f;
    vec2 screenSunPosition = vec2(-10.0);
    lf1Pos = lf2Pos = lf3Pos = lf4Pos = vec2(-10.0);
    if(ndcSunPosition.x >= -1.0 && ndcSunPosition.x <= 1.0 &&
        ndcSunPosition.y >= -1.0 && ndcSunPosition.y <= 1.0 &&
        ndcSunPosition.z >= -1.0 && ndcSunPosition.z <= 1.0)
    {
        screenSunPosition = ndcSunPosition.xy * 0.5 + 0.5;
        for(int x = -4; x <= 4; x++)
        {
            for(int y = -4; y <= 4; y++)
            {
                float depth = textureLod(depthtex0, screenSunPosition.st + vec2(x, y) * screenTexel, 0.0).r;
                if(depth > 0.9999)
                    sunVisibility += 1.0 / 81.0;
            }
        }
        float shortestDis = min( min(screenSunPosition.s, 1.0 - screenSunPosition.s),
                                min(screenSunPosition.t, 1.0 - screenSunPosition.t));
        sunVisibility *= smoothstep(0.0, 0.2, clamp(shortestDis, 0.0, 0.2));
        
        vec2 dir = vec2(0.5) - screenSunPosition;
        lf1Pos = vec2(0.5) + dir * LF1POS;
        lf2Pos = vec2(0.5) + dir * LF2POS;
        lf3Pos = vec2(0.5) + dir * LF3POS;
        lf4Pos = vec2(0.5) + dir * LF4POS;
    }
    #endif
}