#version 460 core

layout(location = 0) out vec4 texBuffer0;

layout(location = 1) out vec4 texBuffer2;

in vec2 Texcoord;

uniform int isEyeInWater;
uniform float far;
uniform vec3 fogColor;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex0;
uniform sampler2D colortex0;
uniform sampler2D colortex1;

float drawLightmap(in float lightmap, const float offset)
{
    float mist = exp(offset);
    return (exp(lightmap * offset) - mist) / (1.0 - mist);
}

vec3 torchColorComp(in float lightmap)
{
    float R = drawLightmap(lightmap, -4.0);
    float G = drawLightmap(lightmap, -7.0);
    float B = drawLightmap(lightmap, -16.0);
    return vec3(R, G, B) * 3.0;
}

void main()
{
    vec4 outColor = texture(colortex0, Texcoord);
    vec2 dataPack = texture(colortex1, Texcoord).xy;
    vec3 linearFogCol = pow(fogColor, vec3(2.2));

    vec4 viewPos = gbufferProjectionInverse * vec4(Texcoord * 2.0 - 1.0, texture(depthtex0, Texcoord).x * 2.0 - 1.0, 1.0);
    viewPos /= viewPos.w;

    outColor.rgb *= (normalize(linearFogCol + 0.0001) * 0.5 + 0.5) * 0.4 * dataPack.y + torchColorComp(1.0 - dataPack.x);
    outColor.rgb = mix(linearFogCol * 1.5, outColor.rgb, exp(-length(viewPos.xyz) / 64.0 * (isEyeInWater == 0 ? 2.0 : 16.0)));
    texBuffer0 = outColor;
    texBuffer2 = outColor * outColor * 0.4;
}
/* DRAWBUFFERS:02 */
/*
const int colortex0Format = RGB16F;
const int colortex1Format = RG8;
const int colortex2Format = RGB16F;
*/