#version 460 core

out vec4 texBuffer2;

in vec2 Texcoord;

uniform vec2 screenTexel;
uniform sampler2D colortex2;

vec3 ColorFetch(vec2 coord)
{
 	return texture(colortex2, coord).rgb;
}

vec3 Grab1(vec2 coord, const float octave, const vec2 offset)
{
 	float scale = exp2(octave);
    
    coord += offset;
    coord *= scale;
    return ColorFetch(coord) * float(abs(coord.x * 2.0 - 1.0) < 1.0 && abs(coord.y * 2.0 - 1.0) < 1.0);
}

vec3 GrabN(vec2 coord, const float octave, const vec2 offset, const int oversampling)
{
 	float scale = exp2(octave);
    
    coord += offset;
    coord *= scale;

   	if (abs(coord.x * 2.0 - 1.0) > 1.0 || abs(coord.y * 2.0 - 1.0) > 1.0)
     	return vec3(0.0);   
    
    vec3 color = vec3(0.0);
    int weights = 0;
    
    for (int i = 0; i < oversampling; i++)
    {
        for (int j = 0; j < oversampling; j++)
        {
            color += ColorFetch(coord + (vec2(i, j) * screenTexel) * scale / float(oversampling));
            weights++;
        }
    }
    
    color /= weights;
    
    return color;
}

vec2 CalcOffset(float octave)
{
    vec2 offset = vec2(0.0);
    
    vec2 padding = screenTexel * 10.0;
    
    offset.x = -min(1.0, floor(octave / 3.0)) * (0.25 + padding.x);
    
    offset.y = -(1.0 - (1.0 / exp2(octave))) - padding.y * octave;

	offset.y += min(1.0, floor(octave / 3.0)) * 0.35;
    
 	return offset;   
}

void main()
{
    vec2 uv = Texcoord;
    
    
    vec3 color = vec3(0.0);
    
    color += Grab1(uv, 1.0, vec2(0.0,  0.0));
    color += GrabN(uv, 2.0, vec2(CalcOffset(1.0)), 4);
    color += GrabN(uv, 3.0, vec2(CalcOffset(2.0)), 8);
    color += GrabN(uv, 4.0, vec2(CalcOffset(3.0)), 16);
    color += GrabN(uv, 5.0, vec2(CalcOffset(4.0)), 16);
    color += GrabN(uv, 6.0, vec2(CalcOffset(5.0)), 16);
    color += GrabN(uv, 7.0, vec2(CalcOffset(6.0)), 16);
    color += GrabN(uv, 8.0, vec2(CalcOffset(7.0)), 16);

    texBuffer2 = vec4(color, 1.0);
}
/* DRAWBUFFERS:2 */