#version 460 compatibility

layout(location = 0) out vec4 texBuffer0;
layout(location = 1) out vec4 texBuffer1;
layout(location = 2) out vec4 texBuffer2;

uniform float frameTimeCounter;
uniform vec3 cameraPosition;

uniform vec3 skyTopColor;
uniform vec3 skyAddColor;
uniform vec3 lightColor;
uniform sampler2D tex;
uniform sampler2D gaux1;

#ifdef MC_NORMAL_MAP
    uniform sampler2D normals;
#endif

#ifdef MC_SPECULAR_MAP
    uniform sampler2D specular;
#endif

#if MC_VERSION >= 11700
    uniform float alphaTestRef;
#endif

flat in float matID;
in float ao;
in vec2 Texcoord;
in vec2 Imcoord;
in vec3 vColor;
flat in vec3 vTangent;
flat in vec3 vBitangent;
flat in vec3 vNormal;
in vec3 worldPos;
in vec3 TexelPos;

mat3 TBNMatrix = {vTangent, vBitangent, vNormal};

#include "/HmwsLib/PackDatas.glsl"

#define WATER_WAVE
#define WATER_PARALLAX
#define waterMapHeight 1.0//[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9]

#ifdef WATER_WAVE
float getWaves(in vec2 position)
{
    position *= 0.009765625;
    float t = frameTimeCounter * 0.0234375;
    float wav1 = texture(gaux1, position + t).x;
    float wav2 = texture(gaux1, position * vec2(3.0, 1.5) + t * vec2(-1.0, 0.5)).x;
    float wav3 = texture(gaux1, position * vec2(6.0, 3.0) - t * vec2(0.5, 0.25)).x;
    return (wav1 * 0.571428571429 + wav2 * 0.285714285714 + wav3 * 0.142857142857) * 1.5 * waterMapHeight;
    // float w1 = texture(gaux1, position + t).x;
    // float w2 = texture(gaux1, position - t).x;
    // return (sin(w1 + w2) * 0.5 + 0.5) * 1.5 * waterMapHeight;
    //return cos(texture(gaux1, position + t).x * 9.19810 + sin((position.x + position.y - t) * 50.514) + cos(position.x * 30.176) + frameTimeCounter) * 0.2 * waterMapHeight;
}

#ifdef WATER_PARALLAX
vec2 ParallaxWaterCoord(vec2 position)
{
    vec2 coord = TexelPos.xy / length(worldPos);
    position += coord * (1.0 - getWaves(position));
    position += coord * (1.0 - getWaves(position));
    position += coord * (1.0 - getWaves(position));
    position += coord * (1.0 - getWaves(position));
    return position;
}
#endif

vec4 getNormals(in vec2 position)
{
    #ifdef WATER_PARALLAX
    position = ParallaxWaterCoord(position);
    #endif
    float h1 = getWaves(position);
    float c1 = getWaves(position + vec2(0.2, 0.0)) - h1;
    float c2 = getWaves(position + vec2(0.0, 0.2)) - h1;
    float delta = abs(c1 + c2);
    delta = 1.0 - delta * delta;
    return vec4(normalize(vec3(c1, c2, delta)), h1);
}
#endif

void main()
{
    GbufferData buff;
    buff.maskID = 1;
    buff.materialID = int(matID + 0.001);
    buff.ao = pow(ao, 2.2);
    buff.Normal = vNormal;
    buff.Albedo = texture(tex, Texcoord);
    #if MC_VERSION >= 11700
        if(buff.Albedo.a < alphaTestRef)
            discard;
    #endif
    buff.Albedo.rgb *= vColor * (skyTopColor + skyAddColor * 0.15 + lightColor * Imcoord.y * Imcoord.y * Imcoord.y * Imcoord.y);
    if(int(matID + 0.0001) == 4)
    {
        buff.Albedo = vec4(0.0, 0.0, 0.0, 0.5);
        #ifdef WATER_WAVE
        vec4 waterNormal = getNormals(worldPos.xz + worldPos.y + cameraPosition.xz + cameraPosition.y);
        buff.Normal = mat3(vTangent, vBitangent, vNormal) * waterNormal.xyz;
        buff.Normal = normalize(mix(buff.Normal, vNormal, pow(1.0 - abs(normalize(TexelPos).z), 5.0)));
        buff.ao = waterNormal.w / 1.75;
        #endif
    }
    buff.Imcoord = Imcoord;
    PackGbufferData(texBuffer0, texBuffer1, texBuffer2, buff);
}
/* DRAWBUFFERS:312 */