#version 460 core

out vec4 texBuffer3;

uniform int isEyeInWater;
uniform int frameCounter;
uniform float flag_day;
uniform float flag_night;
uniform float flag_sunset;

uniform float rainStrength;
uniform float near;
uniform float far;
uniform vec3 sunVec;
uniform vec3 moonVec;
uniform vec3 shadowVec;
uniform vec3 upVec;
uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D colortex6;
uniform sampler2DShadow shadowtex1;

in vec2 Texcoord;

#include "/HmwsLib/BasicFunctions.glsl"
#include "/HmwsLib/PackDatas.glsl"
#include "/HmwsLib/SpaceTransform.glsl"
#include "/HmwsLib/EndSky.glsl"
#include "/HmwsLib/RayTrac.glsl"
#include "/HmwsLib/Dithering.glsl"

float DistributionGGX(vec3 N, vec3 H, float roughness)
{
    float a      = roughness*roughness;
    float a2     = a*a;
    float NdotH  = max(dot(N, H), 0.0);
    float NdotH2 = NdotH*NdotH;

    float nom   = a2;
    float denom = (NdotH2 * (a2 - 1.0) + 1.0);
    denom = 3.14159265 * denom * denom;

    return nom / denom;
}

void main()
{
    vec4 pack0 = texture(colortex0, Texcoord);
    vec4 pack1 = texture(colortex1, Texcoord);
    vec4 pack2 = texture(colortex2, Texcoord);
    GbufferData gbuffer;
    ReadGbufferData(pack0, pack1, pack2, gbuffer);
    vec4 outColor = texture(colortex3, Texcoord);
    
    vec3 viewPos0 = getView(Texcoord, 0);
    vec3 viewPos1 = getView(Texcoord, 1);
    vec3 worldPos0 = View2World(viewPos0);
    float worldLen = sqrt(pow2(viewPos0.x) + pow2(viewPos0.y) + pow2(viewPos0.z));
    vec3 viewDir = viewPos0 / worldLen;
    vec3 worldDir = worldPos0 / worldLen;
    float dither = calcDither();
    if(gbuffer.materialID == 4)
    {
        vec2 warp = (vec2(gbuffer.ao, 1.0 - gbuffer.ao) * 2.0 - 1.0) * (1.0 - exp(-distance(viewPos0, viewPos1)));
        outColor.r = texture(colortex3, Texcoord + warp * 0.05).r;
        outColor.g = texture(colortex3, Texcoord + warp * 0.06).g;
        outColor.b = texture(colortex3, Texcoord + warp * 0.07).b;

        vec3 refDir = reflect(viewDir, gbuffer.Normal);
        vec3 refWorldDir = viewMat3Inverse * refDir;
        //float dither = fract(hash(gl_FragCoord.xy) + float(frameCounter) / 4.1672575);
        bool rr = true;
        #ifdef SSR
        vec3 refCoord = RayTracTexcoord(viewPos0 + gbuffer.Normal * (-viewPos0.z / far * 0.4 + 0.05), refDir, dither);
        rr = refCoord.z < 0.5;
        #endif
        vec3 refCol;

        if(rr)
        {
            refCol = DrawEndSky(refWorldDir, dither, true);
        }else{
        #ifdef SSR
            refCol = texture(colortex3, refCoord.xy).rgb;
        #endif
        }
        float cosTheta = dot(-viewDir, gbuffer.Normal);
        float fakeFresnel = pow5(1.0 - cosTheta);
        fakeFresnel = isEyeInWater == 0 ? fakeFresnel * 0.96 + 0.04 : smoothstep(0.01, 0.175, fakeFresnel);
        outColor.rgb = mix(outColor.rgb, refCol, fakeFresnel);

        
        vec3 shadowPos = World2ShadowProj(worldPos0);
        
        float NdotL = dot(gbuffer.Normal, shadowVec);
        float sdist = length(shadowPos.xy * 2.0 - 1.0);
        float offset = 8e-4 / (1.0 - sdist) * sqrt(1.0 - NdotL * NdotL);
        shadowPos.z -= offset;

        float highlightmask = textureLod(shadowtex1, shadowPos, 1.0) * min(dot(shadowVec, upVec) * 10.0, 1.0);

        outColor.rgb += vec3(1.0, 0.75, 1.0) * min(DistributionGGX(gbuffer.Normal, normalize(-viewDir - (viewMat3 * shadowModelView[1].xyz)), 0.125), 6.24) * highlightmask;

    }

    if(gbuffer.maskID == 1 && isEyeInWater == 0)
    {
        vec3 fogCol = DrawEndSky(worldDir, dither, false);
        outColor.rgb = mix(fogCol, outColor.rgb, exp(worldLen * -0.01));
    }

    texBuffer3 = outColor;
}
/* DRAWBUFFERS:3 */