#version 460 core

out vec4 texBuffer3;

const float sunPathRotation = 35.0;//[0.0 5.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 45.0 50.0 55.0]
const float ambientOcclusionLevel = 1.0;//[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
const int shadowMapResolution = 2048;//[1024 2048 4096 8912]
const float shadowDistance = 200.0;//[128.0 192.0 256.0 320.0 384.0 448.0 512.0 576.0 640.0 704.0 768.0 832.0 896.0 960.0 1024.0]
const bool shadowHardwareFiltering = true;

const bool shadowtex0Mipmap = true;
const bool shadowtex1Mipmap = true;

uniform int frameCounter;
uniform float rainStrength;
uniform float aspectRatio;
uniform float flag_day;
uniform float flag_night;
uniform float flag_sunset;
uniform float near;
uniform float far;
uniform vec3 sunVec;
uniform vec3 moonVec;
uniform vec3 shadowVec;
uniform vec3 upVec;
uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D shadowtex0;
uniform sampler2DShadow shadowtex1;

in vec2 Texcoord;

#include "/HmwsLib/BasicFunctions.glsl"
#include "/HmwsLib/PackDatas.glsl"
#include "/HmwsLib/SpaceTransform.glsl"
#include "/HmwsLib/Dithering.glsl"
#include "/HmwsLib/EndSky.glsl"

const vec2 shadowOffsets[9] =
{
    vec2( 0.0, 0.0),
    vec2( 0.0, 1.0),
    vec2( 0.7, 0.7),
    vec2( 1.0, 0.0),
    vec2( 0.7, -0.7),
    vec2( 0.0, -1.0),
    vec2(-0.7, -0.7),
    vec2(-1.0, 0.0),
    vec2(-0.7, 0.7)
};

#define SHADOW_AA

float SSAAShadow(in vec3 shadowPos)
{
    #ifdef SHADOW_AA
        float value = 0.0;
        const float a = 1.0 / shadowMapResolution;
        const float b = 0.7 * a;
        value += texture(shadowtex1, shadowPos);
        value += texture(shadowtex1, shadowPos + vec3(0.0, a, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(b, b, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(a, 0.0, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(b, -b, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(0.0, -a, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(-b, -b, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(-a, 0.0, 0.0));
        value += texture(shadowtex1, shadowPos + vec3(-b, b, 0.0));
        return value / 9;
    #else
        return texture(shadowtex1, shadowPos);
    #endif
}

float drawLightmap(in float lightmap, const float offset)
{
    float mist = exp(offset);
    return (exp(lightmap * offset) - mist) / (1.0 - mist);
}

vec3 torchColorComp(in float lightmap)
{
    float R = drawLightmap(lightmap, -4.0);
    float G = drawLightmap(lightmap, -7.0);
    float B = drawLightmap(lightmap, -16.0);
    return vec3(R, G, B) * 3.0;
}

float ExpToLinearDepth(in float depth)
{
    return (2.0 * near) / (far + near - depth * (far - near));
}

#define SSAO_ENABLE
#define SSAO_SAMPLES 6//[4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32]
#define SSAO_RADIUS 1.0//[0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8]

#ifdef SSAO_ENABLE

vec3 randomKernel(in float s)
{
    return vec3(fract(sin(s) * 15318.1629762) * 2.0 - 1.0, fract(cos(s) * 16927.61926) * 2.0 - 1.0, 0.0) * SSAO_RADIUS;
}

float SSAO(vec3 viewPos, vec3 normal, float dither)
{
    if(texture(depthtex1, Texcoord).x < 0.56)
        return 1.0;
    float result = 0.0;
    float s = dither * 1.6215865;
    for(int i = 0;i < SSAO_SAMPLES;i++)
    {
        s += 1.6215865;
        vec3 samples = View2Proj(viewPos + normalize(randomKernel(s) + normal)) * 0.5 + 0.5;
        result += clamp((ExpToLinearDepth(texture(depthtex1, samples.xy).x) - ExpToLinearDepth(samples.z)) * 64.0, 0.0, 1.0);
    }
    return result / SSAO_SAMPLES;
}
    
#endif

void main()
{
    vec4 pack0 = texture(colortex0, Texcoord);
    vec4 pack1 = texture(colortex1, Texcoord);
    vec4 pack2 = texture(colortex2, Texcoord);
    GbufferData gbuffer;
    ReadGbufferData(pack0, pack1, pack2, gbuffer);

    vec3 viewPos1 = getView(Texcoord, 1);
    vec3 worldPos1 = View2World(viewPos1);
    float worldLen = length(worldPos1);
    vec3 worldDir = worldPos1 / worldLen;
    vec3 viewDir = viewPos1 / worldLen;
    vec4 outColor;
    float dither = calcDither();
    if(gbuffer.maskID == 0)
    {
        outColor.rgb = DrawEndSky(worldDir, dither, true);
    }else{
        
        vec3 shadowPos = World2ShadowProj(worldPos1);
        vec3 ssPos = shadowPos;
        float NdotL = dot(gbuffer.Normal, viewMat3 * -shadowModelView[1].xyz);
        float sdist = length(shadowPos.xy * 2.0 - 1.0);
        float offset = 8e-4 / (1.0 - sdist) * sqrt(1.0 - NdotL * NdotL);
        shadowPos.z -= offset;
        float shad = SSAAShadow(shadowPos);
        float ss = shad;
        float isplant = float(gbuffer.materialID == 2 || gbuffer.materialID == 3);

        float fadesun = min(-shadowModelView[1].y * 8.0, 1.0);
        shad *= (clamp(NdotL * 3.0 - 1.0, 0.0, 1.0) * (1.0 - isplant) + isplant) * fadesun;
        outColor = gbuffer.Albedo;

        //float ao = gbuffer.ao * (1.0 - shad) + shad;
        #ifdef SSAO_ENABLE
        float ao = SSAO(viewPos1, gbuffer.Normal, dither) * (1.0 - shad) + shad;
        #else
        float ao = gbuffer.ao * (1.0 - shad) + shad;
        #endif
        vec3 ambient = (skyTopColor + skyAddColor * 0.2) * ao;
        vec3 light = vec3(0.2, 0.15, 0.25) * shad * ao;
        vec3 polight = torchColorComp(1.0 - gbuffer.Imcoord.x);
        vec3 highlight = vec3(0.2, 0.15, 0.2) * ss * pow16(max(dot(normalize(-worldDir - shadowModelView[1].xyz), viewMat3Inverse * gbuffer.Normal), 0.0)) * fadesun;
        outColor.rgb *= ambient + light + polight + highlight;
        //outColor.rgb = vec3(SSAO(viewPos1, gbuffer.Normal, dither));
    }
    texBuffer3 = outColor;
}
/* DRAWBUFFERS:3 */
/*
const int colortex0Format = RGB8F;
const int colortex1Format = RGB16;
const int colortex2Format = RGB8;
const int colortex3Format = RGB16F;
const int colortex4Format = RGB8F;
const int colortex5Format = RGB16F;
const int gaux4Format = RGB16;
*/