#version 460 core

out vec4 FinalColor;

uniform int frameCounter;
uniform vec2 screenSize;
uniform vec2 screenTexel;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D depthtex0;

in vec2 Texcoord;

#define TONEMAP ACESFilm//[ACESTonemap ACESTonemap2 HMWSTonemap ToneMap_Hejl2015 HableTonemap ACESFilm Tonemap_Lottes]
//#define FILM_FILTER
#define FILTER_GAMMA 1.2//[0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9]

#include "/HmwsLib/BasicFunctions.glsl"

const mat3 ACESInputMat =
mat3(0.59719, 0.35458, 0.04823,
    0.07600, 0.90834, 0.01566,
    0.02840, 0.13383, 0.83777
);

// ODT_SAT => XYZ => D60_2_D65 => sRGB
const mat3 ACESOutputMat =
mat3( 1.60475, -0.54608, -0.07367,
    -0.10208,  1.10813, -0.00605,
    -0.00327, -0.07276,  1.07602
);

vec3 ACESTonemap(vec3 color){
	color = color * ACESInputMat;
	color = (color * (2.51 * color + 0.03)) / (color * (2.43 * color + 0.59) + 0.14);
	color = color * ACESOutputMat;
	color = clamp(color, 0.0, 1.0);
	return color * 0.78;
}

vec3 RRTAndODTFit(in vec3 v)
{
    return (v * (v + 0.0245786f) - 0.000090537f) / (v * (v + 0.4329510f) + 0.238081f);
}

vec3 ACESTonemap2(vec3 color){
	color *= 1.5;
	color = color * ACESInputMat;
    color = RRTAndODTFit(color);
	color = color * ACESOutputMat;
    color = clamp(color, 0.0, 1.0);
    return color;
}


float GetLuminance(vec3 x)
{
    return dot(x, vec3(0.2126, 0.7152, 0.0722));
}

vec3 HMWSTonemap(vec3 color)
{
    color *= 2.3125;
    return (color * (1.0 - exp(-color)) * 1.25) / (color + 1.0);
}


vec3 ToneMap_Hejl2015(in vec3 hdr){
    vec4 vh = vec4(hdr*0.85, 3.0);	//0
    vec4 va = (1.75 * vh) + 0.05;	//0.05
    vec4 vf = ((vh * va + 0.004f) / ((vh * (va + 0.55f) + 0.0491f))) - 0.0821f+0.000633604888;	//((0+0.004)/((0*(0.05+0.55)+0.0491)))-0.0821
    return vf.xyz / vf.w;
}


vec3 HableTonemap(vec3 linearColor) {
	const float A = 0.45;
	const float B = 0.28;
	const float C = 0.1;
	const float D = 0.5;
	const float E = 0.01;
	const float F = 0.3;
	vec3 x = linearColor * 2.0;
	vec3 color = ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
	const float W = 11.0;
	const float white = ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
	return color / white;
}


vec3 ACESFilm(vec3 x)
{
	x *= 0.9;
	return (x * (2.51 * x + 0.03)) / (x * (2.43 * x + 0.59) + 0.14);
}

vec3 Tonemap_Lottes(vec3 x) {
    // Lottes 2016, "Advanced Techniques and Optimization of HDR Color Pipelines"
		const float a = 1.6;
    const float d = 0.977;
    const float hdrMax = 8.0;
    const float midIn = 0.23;
    const float midOut = 0.267;

    // Can be precomputed
    const float b =
        (-pow(midIn, a) + pow(hdrMax, a) * midOut) /
        ((pow(hdrMax, a * d) - pow(midIn, a * d)) * midOut);
    const float c =
        (pow(hdrMax, a * d) * pow(midIn, a) - pow(hdrMax, a) * pow(midIn, a * d) * midOut) /
        ((pow(hdrMax, a * d) - pow(midIn, a * d)) * midOut);

    return pow(x,vec3(a)) / (pow(x, vec3(a * d)) * b + c);
}


// const float weight[5] = float[] (0.227027, 0.1945946, 0.1216216, 0.054054, 0.016216);

// vec3 calcBloomData()
// {
//     vec3 result = texture(colortex4, Texcoord).rgb * weight[0];
//     for(int i = 1; i < 5; ++i)
//     {
//         result += texture(colortex4, Texcoord + vec2(0.0, screenTexel.y * i)).rgb * weight[i];
//         result += texture(colortex4, Texcoord - vec2(0.0, screenTexel.y * i)).rgb * weight[i];
//     }
//     return result;
// }

vec4 cubic(float x)
{
    float x2 = x * x;
    float x3 = x2 * x;
    vec4 w;
    w.x =   -x3 + 3.0*x2 - 3.0*x + 1.0;
    w.y =  3.0*x3 - 6.0*x2       + 4.0;
    w.z = -3.0*x3 + 3.0*x2 + 3.0*x + 1.0;
    w.w =  x3;
    return w / 6.0;
}

vec4 BicubicTexture(in sampler2D tex, in vec2 coord)
{
	vec2 resolution = screenSize;

	coord *= resolution;

	float fx = fract(coord.x);
    float fy = fract(coord.y);
    coord.x -= fx;
    coord.y -= fy;

    fx -= 0.5;
    fy -= 0.5;

    vec4 xcubic = cubic(fx);
    vec4 ycubic = cubic(fy);

    vec4 c = vec4(coord.x - 0.5, coord.x + 1.5, coord.y - 0.5, coord.y + 1.5);
    vec4 s = vec4(xcubic.x + xcubic.y, xcubic.z + xcubic.w, ycubic.x + ycubic.y, ycubic.z + ycubic.w);
    vec4 offset = c + vec4(xcubic.y, xcubic.w, ycubic.y, ycubic.w) / s;

    vec4 sample0 = texture(tex, vec2(offset.x, offset.z) / resolution);
    vec4 sample1 = texture(tex, vec2(offset.y, offset.z) / resolution);
    vec4 sample2 = texture(tex, vec2(offset.x, offset.w) / resolution);
    vec4 sample3 = texture(tex, vec2(offset.y, offset.w) / resolution);

    float sx = s.x / (s.x + s.y);
    float sy = s.z / (s.z + s.w);

    return mix( mix(sample3, sample2, sx), mix(sample1, sample0, sx), sy);
}

//#define BETTER_BLOOM
#ifdef BETTER_BLOOM
vec3 calcBloomData()
{
    vec3 bloom;
    bloom += BicubicTexture(colortex4, Texcoord / 2.0).rgb * 0.05;
	bloom += BicubicTexture(colortex4, (Texcoord + vec2(1.0 / 2.0 + screenTexel.x * 10.0, 0.0)) - Texcoord * (3.0 / 4.0)).rgb * 0.1;
	bloom += BicubicTexture(colortex4, (Texcoord + vec2(1.0 / 2.0 + 1.0 / 4.0 + screenTexel.x * 20.0, 0.0)) - Texcoord * (7.0 / 8.0)).rgb * 0.3;
    bloom += BicubicTexture(colortex4, (Texcoord + vec2(1.0 / 2.0 + 1.0 / 4.0 + 1.0 / 8.0 + screenTexel.x * 30.0, 0.0)) - Texcoord * (15.0 / 16.0)).rgb * 0.6;
    return bloom / 0.95;
}
#endif

vec3 hash23(in vec2 p)
{
	const float C = 65536.0 / 3.14159265;
	vec4 v = (1.0 / 65536.0) * p.xyxy + vec4(0.1, 0.2, 0.3, 0.4);

	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);
	v = fract(v.yzwx * C + v.xyzw);

	return v.xyz * 2.0 - 1.0;
}

void main()
{
    vec4 outColor = texture(colortex3, Texcoord);
    //outColor.rgb += calcBloomData();
    #ifdef BETTER_BLOOM
    outColor.rgb += calcBloomData();
    //outColor = texture(colortex4, Texcoord);
    #else
    outColor.rgb += texture(colortex4, Texcoord).rgb;
    #endif

    outColor.rgb = clamp(outColor.rgb, 0.0, 1.5);
    //outColor.rgb = mix(vec3(GetLuminance(outColor.rgb)), pow(outColor.rgb, vec3(1.0, 0.9, 0.86)), 1.25);
    outColor.rgb *= normalize(outColor.rgb * vec3(1.0, 1.1, 1.2)) * 0.8 + 0.7;

    #ifdef FILM_FILTER
    outColor.rgb *= float(abs(Texcoord.y * 2.0 - 1.0) < 0.9 || texture(depthtex0, Texcoord).x < 0.56);
    outColor.rgb *= (1.0 - length(Texcoord * 2.0 - 1.0) / 1.414) * 0.9 + 0.1;
    #endif

    outColor.rgb = TONEMAP(outColor.rgb);
    outColor.rgb = pow(length(outColor.rgb), 1.0 / FILTER_GAMMA) * normalize(outColor.rgb + 0.0001);

    outColor.rgb = pow(outColor.rgb, vec3(1.0 / 2.2));
    outColor.rgb += hash23(gl_FragCoord.xy) * (2.0 / 255.0);
    
    FinalColor = outColor;    
}