#version 460 core

layout(location = 0) out vec4 texBuffer3;
layout(location = 1) out vec4 texBuffer5;

uniform int frameCounter;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform float far;
uniform float near;
uniform vec2 screenSize;
uniform vec2 screenTexel;
uniform vec2 taaOffset;
uniform sampler2D gaux4;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D colortex5;

const bool colortex5Clear = false;

in vec2 Texcoord;

#include "/HmwsLib/BasicFunctions.glsl"
#include "/HmwsLib/SpaceTransform.glsl"

#define TAA

#ifdef TAA

const vec2 prevCoordOffset[8] =
{
	vec2(1.0, 1.0),
	vec2(1.0, 0.0),
	vec2(1.0, -1.0),
	vec2(0.0, 1.0),
	vec2(0.0, -1.0),
	vec2(-1.0, 1.0),
	vec2(-1.0, 0.0),
	vec2(-1.0, -1.0)
};

float GetLuminance(in vec3 x)
{
    return dot(x, vec3(0.2126, 0.7152, 0.0722));
}

vec3 CalcDvUV(vec2 uv, vec2 offset)
{
	vec2 wavUv = uv + offset;
	return vec3(wavUv, texture(depthtex0, wavUv).x);
}

vec3 CalcClosestToCamera(vec2 uv)
{
	vec3 d0 = CalcDvUV(uv, -screenTexel);
	vec3 d1 = CalcDvUV(uv, screenTexel);
	vec3 d2 = CalcDvUV(uv, vec2(screenTexel.x, -screenTexel.y));
	vec3 d3 = CalcDvUV(uv, vec2(-screenTexel.x, screenTexel.y));
	vec3 dc = CalcDvUV(uv, vec2(0.0));
	dc = dc.z < d0.z ? dc : d0;
	dc = dc.z < d1.z ? dc : d1;
	dc = dc.z < d2.z ? dc : d2;
	dc = dc.z < d3.z ? dc : d3;
	return dc;
}

vec3 colorMixSpace(vec3 color)
{
	return color / (1.0 + GetLuminance(color));
}

vec3 colorMixSpaceInverse(vec3 color)
{
	return color / (1.0 - GetLuminance(color));
}

void DoTAA(inout vec3 color)
{
	vec3 closestToCamera = CalcClosestToCamera(Texcoord);
	vec3 viewPos = Proj2View(closestToCamera * 2.0 - 1.0);
	vec3 worldPos = View2World(viewPos);
	vec3 prevWorldPos = worldPos + (cameraPosition - previousCameraPosition)* float(closestToCamera.z > 0.56 && closestToCamera.z < 1.0);
	vec3 prevViewPos =  PreWorld2PreView(prevWorldPos);
	vec3 prevProjPos = PreView2PreProj(prevViewPos) * 0.5 + 0.5;

	vec2 velocity = prevProjPos.xy - closestToCamera.xy;
	prevProjPos.xy = Texcoord + velocity;

	if (prevProjPos.x < 0.0 || prevProjPos.y < 0.0 || prevProjPos.x > 1.0 || prevProjPos.y > 1.0)
	{
		color = texture(colortex3, Texcoord + taaOffset * 0.5).xyz;
		return;
	}

	vec3 maxc = color;
	vec3 minc = color;

	vec3 samp = texture(colortex3, Texcoord + screenTexel).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(0.0, screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(-screenTexel.x, screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(screenTexel.x, 0.0)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(-screenTexel.x, 0.0)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(screenTexel.x, -screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(0.0, -screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord - screenTexel).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);

	color = texture(colortex3, Texcoord + taaOffset * 0.5).rgb;

	vec3 tempColor = max(texture(colortex5, prevProjPos.xy).rgb, 0.0);
	vec3 finalColor = clamp(tempColor, minc, maxc);

	float isclamped = distance(tempColor, finalColor) / GetLuminance(tempColor) * 0.5;
	float movementRejection = (0.12 + isclamped) * clamp(length(velocity * screenSize), 0.0, 1.0) * float(closestToCamera.z > 0.56 && closestToCamera.z < 1.0);

	color = mix(finalColor, color, clamp(0.1 + movementRejection, 0.0, 1.0));
}
#endif

//#define MOTION_BLUR
#define MOTION_BLUR_QUALITY 8

#ifdef MOTION_BLUR

void motionBlur(inout vec3 color)
{
	vec2 move = (prvCoord - Texcoord);
	float len = min(length(move) / MOTION_BLUR_QUALITY * 0.2, 0.1);
	vec2 dir = normalize(move) * len;
	vec2 point = Texcoord;
	if(len > 0.0)
	{
		for(int i = 0;i < MOTION_BLUR_QUALITY;i++)
		{
			point += dir;
			color += texture(colortex3, point).rgb;
		}
		color /= MOTION_BLUR_QUALITY + 1;
	}
}	
#endif

void main()
{
    vec4 outColor = texture(colortex3, Texcoord);
	
	#ifdef MOTION_BLUR
    motionBlur(outColor.rgb);
	#endif
	#ifdef TAA
    	DoTAA(outColor.rgb);
	#endif
	
    texBuffer3 = outColor;
    
    texBuffer5 = vec4(outColor.rgb, 1.0);
}
/* DRAWBUFFERS:35 */